"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getClass = void 0;
const xregexp_1 = __importDefault(require("xregexp"));
const fields_1 = require("./fields");
const methods_1 = require("./methods");
const modifiers_1 = require("./modifiers");
const cleaner_1 = require("./util/cleaner");
/**
 * extracts class information from a java class code string
 * @param javaContent the content of a java file
 * @param packageName [the name of the package the java file belongs to]
 * @param preAmble [the preamble of a java file (for inner classes)]
 * @param className [the classname, useful for inner classes]
 * @returns the class object
 */
function getClass(javaContent, packageName, preAmble, className) {
    javaContent = (0, cleaner_1.cleanJavaContent)(javaContent);
    // class is an inner class, preamble was already extracted
    if (!preAmble) {
        const result = getPreAmble(javaContent);
        preAmble = result.preAmble;
        javaContent = result.rest;
    }
    packageName = packageName ?? getPackageName(preAmble);
    let classDeclaration;
    let classContent;
    // matches main curlies -> returns classDeclaration and classContent without surrouding curlies
    xregexp_1.default.matchRecursive(javaContent, '\\{', '\\}', 'g', {
        valueNames: ['classDeclaration', null, 'classContent', null],
    }).forEach((matchElement) => {
        if (matchElement.name === 'classDeclaration') {
            classDeclaration = matchElement.value;
        }
        else if (matchElement.name === 'classContent') {
            classContent = matchElement.value;
        }
    });
    className = classDeclaration ? getClassname(classDeclaration) : 'UnknownClass';
    let investigatables = [];
    const classes = [];
    // matches inner curlies -> returns between (actual information)
    // and match(content, but also inner class content)
    let isInnerClass = false;
    const contentMatchArray = xregexp_1.default.matchRecursive(classContent ?? '', '\\{', '\\}', 'g', {
        valueNames: ['between', null, 'match', null],
    });
    for (const matchElement of contentMatchArray) {
        if (matchElement.name === 'between' && matchElement.value.match(/\s(interface|class|enum)\s/)) {
            isInnerClass = matchElement.value;
        }
        else if (isInnerClass && matchElement.name === 'match') {
            const innerJavaContent = `${isInnerClass} { ${matchElement.value} }`;
            const innerClass = getClass(innerJavaContent, packageName, preAmble);
            const nameArray = innerClass.name.split('.');
            nameArray.splice(-1, 1, `${className}$${nameArray.at(-1)}`);
            innerClass.name = nameArray.join('.');
            classes.push(innerClass);
            isInnerClass = false;
        }
        else if (matchElement.name === 'between') {
            investigatables = investigatables.concat(matchElement.value.split(';'));
        }
    }
    // no inner curlies, everything is relevant
    if (contentMatchArray.length === 0) {
        investigatables = classContent?.split(';') ?? [];
    }
    investigatables = investigatables.filter((value) => !value.match(/^\s*$/));
    const fields = (0, fields_1.getFields)(investigatables);
    const methods = (0, methods_1.getMethods)(investigatables);
    const extendsClass = classDeclaration ? getExtends(classDeclaration, preAmble) : undefined;
    const modifiers = classDeclaration ? (0, modifiers_1.getModifiers)(classDeclaration).modifiers : [];
    const implementsClasses = classDeclaration ? getImplements(classDeclaration, preAmble) : [];
    const values = modifiers.includes(modifiers_1.Modifier.enum) ? classContent?.split(',') : undefined;
    return {
        name: `${packageName}.${className}`,
        extends: extendsClass,
        modifiers,
        implements: implementsClasses,
        fields: modifiers.includes(modifiers_1.Modifier.enum) ? [] : fields,
        methods: modifiers.includes(modifiers_1.Modifier.enum) ? [] : methods,
        classes: modifiers.includes(modifiers_1.Modifier.enum) ? [] : classes,
        values: modifiers.includes(modifiers_1.Modifier.enum) ? values : [],
    };
}
exports.getClass = getClass;
/**
 * Extracts the preamble (package declaration and imports) from java code
 * @param javaContent the java code to analyze
 * @returns an object with the preamble and the rest of the java code
 */
function getPreAmble(javaContent) {
    // matches import or package statements,
    // negative lookbehind to check that they are not inside curlies
    const preAmbleMatches = javaContent.matchAll(/(?<!{[\s\S]*)package[^;]*;|(?<!{[\s\S]*)import[^;]*;/g);
    let preAmble = '';
    let rest = javaContent;
    let statement = '';
    while (statement !== undefined) {
        rest = rest.replace(statement, '');
        preAmble += statement;
        statement = preAmbleMatches.next().value;
    }
    const preAmbleArray = preAmble.split(';').filter((value) => value.length > 0);
    return { preAmble: preAmbleArray, rest };
}
/**
 * Extracts the package name from preamble
 * @param preAmble the preamble containing a package declaration
 * @returns the package name or 'unknown.package' if it was not successfull
 */
function getPackageName(preAmble) {
    for (const candidate of preAmble) {
        const packageMatch = candidate.match(/(?<=package\s)[\s\S]*/);
        if (packageMatch)
            return packageMatch[0].replaceAll(/\s/g, '');
    }
    return 'unknown.package';
}
/**
 * Extracts all classnames that the class implements
 * @param classDeclaration the class declaration to analyze
 * @param preAmble the prampble to extract Fully Qualified Names from
 * @returns fully qualified names of the classes that the class implements
 */
function getImplements(classDeclaration, preAmble) {
    const implementsMatchArray = classDeclaration.match(/implements ([\s\S]*)/);
    const implementsClassNames = implementsMatchArray ? implementsMatchArray[1] : undefined;
    if (!implementsClassNames)
        return [];
    const implementsClassNameArray = implementsClassNames.split(',');
    const implementsFQN = [];
    implementsClassNameArray.forEach((implementsClassName) => {
        const fqn = getFQN(preAmble, implementsClassName.trim());
        if (fqn)
            implementsFQN.push(fqn);
    });
    return implementsFQN;
}
/**
 * extract the class the class is extending
 * @param classDeclaration the class declaration to analyze
 * @param preAmble the preamble to extract Fully Qualified Names from
 * @returns the fully qualified name of the class that is being extended
 */
function getExtends(classDeclaration, preAmble) {
    const extendsMatchArray = classDeclaration.match(/extends ([\s\S]*)/);
    const extendsClassName = extendsMatchArray ? extendsMatchArray[1] : undefined;
    if (!extendsClassName)
        return;
    return getFQN(preAmble, extendsClassName.trim());
}
/**
 * Finds the Fully Qualified Name belonging to a class
 * @param preAmble is searched for the Fully Qualified Name
 * @param name the name to search for
 * @returns the fully qualified name belonging to the name
 */
function getFQN(preAmble, name) {
    // for things like EventHandler<InputEvent>
    const innerMatchArray = xregexp_1.default.matchRecursive(name, '<', '>', 'g', {
        valueNames: ['outer', null, 'inner', null],
    });
    name = innerMatchArray[0]?.value ?? name;
    const innerType = innerMatchArray[1]?.name === 'inner' ? innerMatchArray[1].value : undefined;
    const innerFQN = innerType ? getFQN(preAmble, innerType) : undefined;
    let fqn = name;
    for (const candidate of preAmble) {
        const candidateMatch = candidate.match(new RegExp('(?<=import )[a-z0-9.]*\\.' + name));
        if (candidateMatch) {
            fqn = candidateMatch[0] ?? name;
        }
    }
    fqn = innerType ? `${fqn}<${innerFQN}>` : fqn;
    return fqn;
}
/**
 * finds the classname in a class declaration (or enum / interface)
 * @param classDeclaration the class / enum / interface declaration to search
 * @returns the classname
 */
function getClassname(classDeclaration) {
    const nameMatch = classDeclaration.match(/(?:class|enum|interface)\s+(\S+)/);
    return nameMatch && nameMatch[1] ? nameMatch[1] : 'UnknownClass';
}
